package matrices;

import java.util.Arrays;

public class ArrayUtil {
    private ArrayUtil() {
    }

    public static double[] clone(double[] array) {
        return Arrays.copyOf(array, array.length);
    }

    public static double[][] clone(double[][] array) {
        int r = array.length;
        int c = array[0].length;
        double[][] array1 = new double[r][c];
        for (int i = 0; i < r; i++) {
            for (int j = 0; j < c; j++) {
                array1[i][j] = array[i][j];
            }
        }
        return array1;
    }

    /**
     * Zamienia tablicę pojedynczą na macierz
     *
     * @param tabl - tablica do przekształcenia
     * @param cols - liczba kolumn
     * @return zwraca macierz
     */
    public static double[][] oneToTwo(double[] tabl, int cols) {
        int rows = tabl.length / cols;
        double[][] temp = new double[rows][cols];
        for (int i = 0; i < tabl.length; i++) {
            temp[i / cols][i % cols] = tabl[i];
        }
        return temp;
    }

    /**
     * Zamienia macierz na tablicę
     *
     * @param tabl - macierz do przekształcenia
     * @return - zwraca tablicę
     */
    public static double[] twoToOne(double[][] tabl) {
        int row = tabl.length;
        int col = tabl[0].length;
        int len = row * col;
        double[] temp = new double[len];
        int k = 0;
        for (double[] doubles : tabl) {
            for (int j = 0; j < col; j++) {
                temp[k] = doubles[j];
                k++;
            }
        }
        return temp;
    }

    public static double[] shortenArray(double[] tabl, int index1, int index2) {
        int len = tabl.length;
        int lenAdd = index2 - index1;
        if (index1 > index2) {
            throw new IllegalArgumentException(
                    "index1 nie może być większy od index2");
        }
        //sprawdza czy indeksy mieszczą się w tablicy
        if ((index2 > len - 1) || (index1 < 0)) {
            throw new IndexOutOfBoundsException("indeks poza tablicą");
        } else {//jeśli tak
            //to przepisuje rekordy za indeksem o 1 w lewo
            for (int i = index1; i < tabl.length - lenAdd - 1; i++) {
                tabl[i] = tabl[i + lenAdd + 1];
            }
        }
        //a następnie skraca tablicę o niezbędną długość
        return Arrays.copyOf(tabl, len - lenAdd - 1);
    }


    public static double[] shortenArray(double[] tabl, int indexDeleted) {
        int len = tabl.length;
        //sprawdza czy index mieści sie w tablicy
        if ((indexDeleted > len - 1) || (indexDeleted < 0)) {
            throw new IndexOutOfBoundsException("index poza tablicą");
        } else {//jeśli tak
            //to przepisuje rekordy za indeksem o 1 w lewo
            for (int i = indexDeleted; i < tabl.length - 1; i++) {
                tabl[i] = tabl[i + 1];
            }
        }
        //a następnie skraca tablicę o 1
        return Arrays.copyOf(tabl, len - 1);
    }

    public static void print(double[] array) {
        System.out.print("[");
        for (int i = 0; i < array.length; i++) {
            if (i == array.length - 1) {
                System.out.print(array[i]);
            } else {
                System.out.print(array[i] + ", ");
            }
        }
        System.out.print("]");
        System.out.println();
    }


    //przelicza indeks z [][] na []
    public static int index(int row, int col, int cols) {
        return row * cols + col;
    }


    public static double[] extendArray(double[] tabl, int oIle) {
        int len = tabl.length;
        return Arrays.copyOf(tabl, len + oIle);
    }

    public static double[] extendArray(double[] tabl, int indexInserted,
                                       double value) {
        int len = tabl.length;
        int lenNew;
        double[] tablNew;
        //sprawdza czy index mieści się w tablicy
        if ((indexInserted > len) || (indexInserted < 0)) {
            throw new IndexOutOfBoundsException("index poza tablicą");
        } else {//jeśli tak
            //przedłuża tablicę
            tablNew = extendArray(tabl);
            lenNew = tablNew.length;
            //to przepisuje rekordy za indeksem o 1 w prawo
            //zaczynając od tylu
            for (int i = lenNew - 1; i > indexInserted; i--) {
                tablNew[i] = tablNew[i - 1];
            }
            tablNew[indexInserted] = value;
        }
        return tablNew;
    }

    public static double[] extendArray(double[] tabl) {
        int len = tabl.length;
        return Arrays.copyOf(tabl, len + 1);
    }

    public static double[] extendArray(double[] tabl, int indexInserted,
                                       double[] tablWstaw) {
        int len = tabl.length;
        int lenWstaw = tablWstaw.length;
        double[] tablNew;
        int lenNew;
        //sprawdza czy index mieści się w tablicy
        if ((indexInserted > len) || (indexInserted < 0)) {
            throw new IndexOutOfBoundsException("index poza tablicą");
        } else {//jeśli tak
            //przedłuża tablicę
            tablNew = extendArray(tabl, lenWstaw);
            lenNew = tablNew.length;
            //to przepisuje rekordy za indeksem o 1 w prawo
            //zaczynając od tyłu
            for (int i = lenNew - 1; i > indexInserted - 1 + lenWstaw; i--) {
                tablNew[i] = tablNew[i - lenWstaw];
            }
            //wstawia tablWstaw w opróżnione miejsce
            for (int k = indexInserted; k < lenWstaw + indexInserted; k++) {
                tablNew[k] = tablWstaw[k - indexInserted];
            }
        }
        return tablNew;
    }

    /**
     * Przedłuża tablicę o 1 element
     */
    public static <T> T[] extendArray(T[] tabl) {
        int len = tabl.length;
        return Arrays.copyOf(tabl, len + 1);
    }

    public static void printToConsole(double[][] tabl, int dec) {
        for (double[] doubles : tabl) {
            for (int j = 0; j < tabl[0].length; j++) {
                System.out
                        .print(MathUtil.roundToDecimal(doubles[j], dec) + " ");
            }
            System.out.println();
        }
    }
}
